﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;

namespace TPCWare.CDays14.Twitter.Service.Model
{
    public class Signature
    {
        private string packedParameters;
        private string signatureBase;
        private string signatureKey;

        /// <summary>
        /// Crteate the Signature helper class.
        /// </summary>
        /// <param name="httpMethod">The http method of the request.</param>
        /// <param name="baseUrl">The base url of the request.</param>
        /// <param name="oauthNonce">The nonce of the request.</param>
        /// <param name="oauthTimestamp">The timestamp of the request.</param>
        /// <param name="oauthConsumerKey">The consumer key.</param>
        /// <param name="oauthConsumerSecret">The consumer secret.</param>
        /// <param name="oauthToken">Can be null if not part of the request.</param>
        /// <param name="oauthTokenSecret">Can be null during user authorization request.</param>
        /// <param name="optionalParameters">Optional parameters (i.e. oauth_callback=www.mysite.com).</param>
        public Signature(string httpMethod, string baseUrl, string oauthNonce, string oauthTimestamp,
                         string oauthConsumerKey, string oauthConsumerSecret,
                         string oauthToken, string oauthTokenSecret,
                         Dictionary<string, string> optionalParameters = null)
        {
            // Percent encode every key and value that will be signed. 
            var urlEncodedParameters = new Dictionary<string, string>();
            urlEncodedParameters.Add(Utils.PercentEncode("oauth_consumer_key"), Utils.PercentEncode(oauthConsumerKey));
            urlEncodedParameters.Add(Utils.PercentEncode("oauth_nonce"), Utils.PercentEncode(oauthNonce));
            urlEncodedParameters.Add(Utils.PercentEncode("oauth_signature_method"), Utils.PercentEncode("HMAC-SHA1"));
            urlEncodedParameters.Add(Utils.PercentEncode("oauth_timestamp"), Utils.PercentEncode(oauthTimestamp));
            if (!string.IsNullOrWhiteSpace(oauthToken))
            {
                urlEncodedParameters.Add(Utils.PercentEncode("oauth_token"), Utils.PercentEncode(oauthToken));
            }
            urlEncodedParameters.Add(Utils.PercentEncode("oauth_version"), Utils.PercentEncode("1.0"));

            // Add optional parameters
            if (optionalParameters != null)
            {
                foreach (var item in optionalParameters)
                {
                    urlEncodedParameters.Add(Utils.PercentEncode(item.Key), Utils.PercentEncode(item.Value));
                }
            }

            // Create the StringBuilder
            StringBuilder sb = new StringBuilder();

            // Create the packed parameters string
            // Note: items keys and values are allready percent encoded so we can pick them ordered by key
            bool firstItem = true;
            foreach (var item in urlEncodedParameters.OrderBy(item => item.Key))
            {
                if (firstItem)
                {
                    firstItem = false;
                }
                else
                {
                    sb.Append('&');
                }
                sb.Append(item.Key);
                sb.Append('=');
                sb.Append(item.Value);
            }
            packedParameters = sb.ToString();

            // Create signature base string
            sb.Clear();
            sb.Append(httpMethod.ToUpper());
            sb.Append('&');
            sb.Append(Utils.PercentEncode(baseUrl));
            sb.Append('&');
            sb.Append(Utils.PercentEncode(packedParameters));
            signatureBase = sb.ToString();

            // Create Signature Key string
            sb.Clear();
            sb.Append(Utils.PercentEncode(oauthConsumerSecret));
            sb.Append('&');
            if (!string.IsNullOrWhiteSpace(oauthTokenSecret))
            {
                sb.Append(Utils.PercentEncode(oauthTokenSecret));
            }
            signatureKey = sb.ToString();
        }

        public string Compute()
        {
            // Create HMAC-SHA1 Hashing instance
            var hmacSha1 = new HMACSHA1(Utils.StringToAscii(signatureKey));

            // Compute hash
            var hash = hmacSha1.ComputeHash(Utils.StringToAscii(signatureBase));

            return Convert.ToBase64String(hash);
        }

#if DEBUG
        public string PackedParameters { get { return packedParameters; } }
        public string SignatureBase { get { return signatureBase; } }
        public string SignatureKey { get { return signatureKey; } }
#endif

    }
}
